@@ -51,7 +51,7 @@ ptxd_kgen_generate_sections()
     local dir
     {
 	for dir in "${kgen_dirs[@]}"; do
-	    if [ \! -d "${dir}" ]; then
+	    if [ ! -d "${dir}" ]; then
 		continue
 	    fi
 	    # '! -name ".#*"' filters out emacs's lock files
@@ -93,7 +93,7 @@ EOF
     local oldIFS="$IFS"
     case "${kgen_part}" in
 	ptx|platform) IFS=: kgen_dirs=( ${PTXDIST_PATH_RULES} ${PTXDIST_PATH_PLATFORMS} ) ;;
-	board|user|collection) return 0 ;;
+	board|setup|localsetup|collection) return 0 ;;
 	*) cat <<EOF
 
 ${PROMPT}error: '${FUNCNAME}' doesn't support '${part}', yet
