@@ -12,6 +12,9 @@ export PTXDIST_CMAKE_TOOLCHAIN_TARGET
 PTXDIST_CMAKE_TOOLCHAIN_HOST="${PTXDIST_GEN_CONFIG_DIR}/toolchain-host.cmake"
 export PTXDIST_CMAKE_TOOLCHAIN_HOST
 
+PTXDIST_CMAKE_TOOLCHAIN_CROSS="${PTXDIST_GEN_CONFIG_DIR}/toolchain-cross.cmake"
+export PTXDIST_CMAKE_TOOLCHAIN_CROSS
+
 #
 # generate cmake toolchain file from template
 #
@@ -22,10 +25,15 @@ export PTXDIST_CMAKE_TOOLCHAIN_HOST
 ptxd_make_cmake_toolchain_target() {
     local sysroot_prefix="${PTXDIST_PATH_SYSROOT_PREFIX}:"
 
+    case "${PTXCONF_ARCH_STRING}" in
+	arm64) CMAKE_ARCH_STRING=aarch64 ;;
+	riscv) CMAKE_ARCH_STRING=riscv64 ;;
+	*) CMAKE_ARCH_STRING=${PTXCONF_ARCH_STRING} ;;
+    esac
     SYSTEM_NAME="Linux" \
 	SYSTEM_VERSION="1" \
 	\
-	SYSTEM_PROCESSOR="${PTXCONF_ARCH_STRING}" \
+	SYSTEM_PROCESSOR="${CMAKE_ARCH_STRING}" \
 	\
 	CC="$(which "${CC}")" \
 	CXX="$(which "${CXX}")" \
@@ -46,11 +54,28 @@ ptxd_make_cmake_toolchain_host() {
     CC="$(which "${CC}")" \
 	CXX="$(which "${CXX}")" \
 	\
-	PREFIX_PATH="${PTXDIST_SYSROOT_HOST}" \
+	PREFIX_PATH="${PTXDIST_SYSROOT_HOST}/usr" \
 	\
-	INCLUDE_DIRECTORIES="${PTXDIST_SYSROOT_HOST}/include" \
-	LINK_DIRECTORIES="${PTXDIST_SYSROOT_HOST}/lib" \
+	INCLUDE_DIRECTORIES="${PTXDIST_SYSROOT_HOST}/usr/include" \
+	LINK_DIRECTORIES="${PTXDIST_SYSROOT_HOST}/usr/lib" \
 	\
 	ptxd_replace_magic "${PTXDIST_TOPDIR}/config/cmake/toolchain-host.cmake.in" > "${1}"
 }
 export -f ptxd_make_cmake_toolchain_host
+
+
+#
+# generate cmake toolchain file from template
+#
+ptxd_make_cmake_toolchain_cross() {
+    CC="$(which "${CC}")" \
+	CXX="$(which "${CXX}")" \
+	\
+	PREFIX_PATH="${PTXDIST_SYSROOT_CROSS}/usr" \
+	\
+	INCLUDE_DIRECTORIES="${PTXDIST_SYSROOT_CROSS}/usr/include" \
+	LINK_DIRECTORIES="${PTXDIST_SYSROOT_CROSS}/usr/lib" \
+	\
+	ptxd_replace_magic "${PTXDIST_TOPDIR}/config/cmake/toolchain-host.cmake.in" > "${1}"
+}
+export -f ptxd_make_cmake_toolchain_cross
