@@ -16,14 +16,24 @@ ptxd_make_world_install_prepare() {
     fi &&
     ptxd_make_world_clean_sysroot &&
     rm -rf -- "${pkg_pkg_dir}" &&
-    mkdir -p -- "${pkg_pkg_dir}"/{etc,{,usr/}{lib,{,s}bin,include,{,share/}{man/man{1,2,3,4,5,6,7,8,9},misc}}} &&
-    if [ "${pkg_type}" != "target" ]; then
+    mkdir -p -- "${pkg_pkg_dir}"/{etc,usr/{lib,{,s}bin,include,{,share/}{man/man{1,2,3,4,5,6,7,8,9},misc}}}
+    if [ "${pkg_type}" = "target" ]; then
+	mkdir -p -- "${pkg_pkg_dir}"/{lib,{,s}bin,include,{,share/}man/man{1,2,3,4,5,6,7,8,9}}
+    else
+	for link in bin sbin lib; do
+	    ln -s "usr/${link}" "${pkg_pkg_dir}/${link}"
+	done
 	ln -s "lib" "${pkg_pkg_dir}/lib64"
+	ln -s "lib" "${pkg_pkg_dir}/usr/lib64"
     fi
 }
 export -f ptxd_make_world_install_prepare
 
 ptxd_make_world_install_python_cleanup() {
+    # The full path to the host-python is used as shebang in generated
+    # executables, so replace it with the correct path.
+    find "${pkg_pkg_dir}"/usr/{bin,sbin}/ -mindepth 1 -maxdepth 1 -print0 | xargs -r -0 \
+	sed -i "1s;^#!/.*/\(python[0-9\.]*\);#!/usr/bin/\1;"
     find "${pkg_pkg_dir}" -type f -name "*.so" -print | while read file; do
 	# Python installs shared libraries with executable flags
 	chmod -x "${file}"
@@ -33,7 +43,7 @@ ptxd_make_world_install_python_cleanup() {
 	    # not python3 or already handled
 	    continue
 	fi
-	cp -v "$(dirname "${file}")/__pycache__/$(basename "${file%py}")"cpython-??.pyc "${file}c" || return
+	cp -v "$(dirname "${file}")/__pycache__/$(basename "${file%py}")"cpython-???.pyc "${file}c" || return
     done &&
     check_pipe_status &&
     find "${pkg_pkg_dir}" -type d -name __pycache__  -print0 | xargs -0 rm -rf &&
@@ -42,6 +52,86 @@ ptxd_make_world_install_python_cleanup() {
 }
 export -f ptxd_make_world_install_python_cleanup
 
+ptxd_make_world_install_python_cleanup_host() {
+    # change shebang to ensure that the python wrapper is used
+    if [[ " ${pkg_build_deps_all} " =~ ' host-python3 ' ]]; then
+	python="$(basename "${ptx_python3_host}")"
+	find "${pkg_pkg_dir}"/usr/{bin,sbin}/ -mindepth 1 -maxdepth 1 -print0 | xargs -0 -r \
+	    sed -i "1s;^#!/.*/\(python[0-9\.]*\);#!/usr/bin/env ${python};"
+    fi
+    if ! [[ " ${pkg_build_deps_all} " =~ ' host-python3 ' ]] && ls "${pkg_pkg_dir}"/usr/lib/system-python3/bin/* &> /dev/null; then
+	mv "${pkg_pkg_dir}"/usr/lib/system-python3/bin/* \
+	    "${pkg_pkg_dir}/usr/bin/"
+    fi
+}
+export -f ptxd_make_world_install_python_cleanup_host
+
+ptxd_make_world_install_python() {
+    local sitepackages python_prefix
+
+    if [[ " ${pkg_build_deps} " =~ ' host-system-python3 ' ]]; then
+	mkdir -p "${pkg_pkg_dir}/usr/lib/system-python3/lib"
+	ln -s lib "${pkg_pkg_dir}/usr/lib/system-python3/lib64"
+    fi
+    if [ -e "${pkg_dir}/pyproject.toml" -a ! -e "${pkg_dir}/setup.py" ] && \
+	    ! [[ " ${pkg_build_deps_all} " =~ ' host-python3-pybuild ' ]]; then
+	if [[ " ${pkg_build_deps_all} " =~ ' host-python3 ' ]]; then
+	    sitepackages="${ptx_python3_sitepackages}"
+	else
+	    # separate path for system Python packages
+	    system_python_major_minor="$(python3 -c 'import sys; print("{}.{}".format(sys.version_info.major, sys.version_info.minor))')"
+	    sitepackages="/usr/lib/system-python3/lib/python${system_python_major_minor}/site-packages"
+	fi
+	cmd=( \
+	    mkdir -p "${pkg_pkg_dir}${sitepackages}" '&&' \
+	    unzip -d "${pkg_pkg_dir}${sitepackages}" \
+	    "${pkg_build_dir}/*.whl"
+	)
+    elif [ -e "${pkg_dir}/pyproject.toml" ] &&
+	    ( [[ " ${pkg_build_deps} " =~ ' host-python3-pybuild ' ]] ||
+	    [[ " ${pkg_build_deps} " =~ ' host-system-python3-pybuild ' ]] ) ; then
+	if [[ " ${pkg_build_deps} " =~ ' host-system-python3 ' ]]; then
+	    python_prefix=/usr/lib/system-python3
+	else
+	    python_prefix=/usr
+	fi
+	cmd=( \
+	    "${pkg_path}" \
+	    "${pkg_env}" \
+	    "${pkg_make_env}" \
+	    "${pkg_install_env}" \
+	    "${ptx_build_python}" \
+	    -m installer \
+	    --prefix="${python_prefix}" \
+	    "--destdir=${pkg_pkg_dir}" \
+	    "${pkg_build_dir}/*.whl"
+	)
+    else
+	cmd=( \
+	    cd "${pkg_build_dir}" '&&' \
+	    "${pkg_path}" \
+	    "${pkg_env}" \
+	    "${pkg_make_env}" \
+	    "${pkg_install_env}" \
+	    "${ptx_build_python}" \
+	    setup.py \
+	    "${pkg_install_opt}" \
+	)
+    fi
+    if [ "${pkg_type}" = target ]; then
+	cmd+=( \
+	    '&&' \
+	    ptxd_make_world_install_python_cleanup \
+	)
+    else
+	cmd+=( \
+	    '&&' \
+	    ptxd_make_world_install_python_cleanup_host \
+	)
+    fi
+}
+export -f ptxd_make_world_install_python
+
 #
 # FIXME: kick ${pkg_install_env}
 #
@@ -71,20 +161,7 @@ ptxd_make_world_install() {
 
     case "${pkg_build_tool}" in
 	python*)
-	cmd=( \
-	    cd "${pkg_build_dir}" '&&' \
-	    "${pkg_path}" \
-	    "${pkg_env}" \
-	    "${pkg_make_env}" \
-	    "${pkg_install_env}" \
-	    "${ptx_build_python}" \
-	    setup.py \
-	    "${pkg_install_opt}" \
-	)
-	if [ "${pkg_type}" = target ]; then
-	    cmd[${#cmd[@]}]='&&'
-	    cmd[${#cmd[@]}]=ptxd_make_world_install_python_cleanup
-	fi
+	ptxd_make_world_install_python
 	;;
 	ninja)
 	cmd=( \
@@ -109,6 +186,9 @@ ptxd_make_world_install() {
 	    "${pkg_install_opt}" \
 	)
 	;;
+	cargo)
+	ptxd_bailout "Packages that use cargo require a custom install stage"
+	;;
 	*)
 	cmd=( \
 	    "${pkg_path}" \
@@ -147,7 +227,7 @@ ptxd_make_world_install_unpack() {
 	*)          pkg_prefix="" ;;
     esac &&
 
-    if [ \! -e "${ptx_pkg_dev_dir}/${pkg_pkg_dev}" ]; then
+    if [ ! -e "${ptx_pkg_dev_dir}/${pkg_pkg_dev}" ]; then
 	ptxd_bailout "Internal error: '$(ptxd_print_path ${ptx_pkg_dev_dir}/${pkg_pkg_dev})' does not exist."
     fi &&
     rm -rf -- "${pkg_pkg_dir}" &&
@@ -170,12 +250,21 @@ ptxd_make_world_install_pack() {
     fi &&
 
     # remove empty dirs
-    test \! -e "${pkg_pkg_dir}" || \
+    test ! -e "${pkg_pkg_dir}" || \
 	find "${pkg_pkg_dir}" -depth -type d -print0 | xargs -r -0 -- \
 	rmdir --ignore-fail-on-non-empty -- &&
     check_pipe_status &&
 
-    if [ \! -e "${pkg_pkg_dir}" ]; then
+    # remove usr-merge and lib64 symlinks. They exist in sysroot anyways,
+    # so avoid races when they are overwritten
+    for link in bin sbin lib lib64 usr/lib64; do
+	link="${pkg_pkg_dir}/${link}"
+	if [ -h "${link}" ]; then
+	    rm "${link}"
+	fi || break
+    done &&
+
+    if [ ! -e "${pkg_pkg_dir}" ]; then
 	if [ -e "${pkg_dir}" ]; then
 	    ptxd_warning "PKG didn't install anything to '${pkg_pkg_dir}'"
 	fi
@@ -191,7 +280,7 @@ ptxd_make_world_install_pack() {
     if [ "${pkg_type}" != "target" ]; then
 	find "${pkg_pkg_dir}" -type f -print | while read file; do
 	    if chrpath "${file}" >& /dev/null; then
-		local rel="$(ptxd_abs2rel "$(dirname "${file}")" "${pkg_pkg_dir}/lib")"
+		local rel="$(ptxd_abs2rel "$(dirname "${file}")" "${pkg_pkg_dir}/usr/lib")"
 		chmod +w "${file}" &&
 		if ! chrpath --replace "\${ORIGIN}/${rel}" "${file}" > /dev/null; then
 		    ptxd_bailout "Failed to adjust rpath for '${file}'"
@@ -219,6 +308,13 @@ ptxd_make_world_install_pack() {
 	-e "s:\(-I\|-isystem \)\(${pkg_sysroot_dir}\|${pkg_sysroot_dir_nolink}\)/*\(/include\|/usr/include\):\1@SYSROOT@\3:g" &&
     check_pipe_status &&
 
+    if [ -d "${pkg_pkg_dir}/usr/bin" ]; then
+	find "${pkg_pkg_dir}/usr/bin" -maxdepth 1 -type f -executable -print0 | \
+	    xargs -r -0 \
+	    sed -i "1s;^#!${pkg_sysroot_dir}\(.*\);#!@SYSROOT@\1;" &&
+	    check_pipe_status
+    fi &&
+
     if [ "${pkg_pkg_dev}" != "NO" -a "$(ptxd_get_ptxconf PTXCONF_PROJECT_CREATE_DEVPKGS)" = "y" ]; then
 	tar -c -C "${ptx_pkg_dir}" -z -f "${ptx_pkg_dir}/${pkg_pkg_dev}" "${pkg_pkg_dir##*/}"
     fi
@@ -255,7 +351,7 @@ ptxd_make_world_install_post() {
 	done
     ) | sort -u > "${ptx_state_dir}/${pkg_label}.pkgconfig"
     # do nothing if pkg_pkg_dir does not exist
-    if [ \! -d "${pkg_pkg_dir}" ]; then
+    if [ ! -d "${pkg_pkg_dir}" ]; then
 	return
     fi &&
 
@@ -264,9 +360,15 @@ ptxd_make_world_install_post() {
     find "${pkg_pkg_dir}" ! -type d -name "${pkg_binconfig_glob}" | while read config; do
 	sed -i -e "s:@SYSROOT@:${pkg_sysroot_dir}:g" "${config}" &&
 	if [ "${pkg_type}" = "target" ]; then
-	    cp -P -- "${config}" "${PTXDIST_SYSROOT_CROSS}/bin" || return
+	    cp -P -- "${config}" "${PTXDIST_SYSROOT_CROSS}/usr/bin" || return
 	fi
     done &&
+    if [ -d "${pkg_pkg_dir}/usr/bin" ]; then
+	find "${pkg_pkg_dir}/usr/bin" -maxdepth 1 -type f -executable -print0 | \
+	    xargs -r -0 \
+	    sed -i "1s;^#!@SYSROOT@\(.*\);#!${pkg_sysroot_dir}\1;" &&
+	    check_pipe_status
+    fi
 
     if [ ! -e "${ptx_pkg_dir}/.${pkg_label}" -o -h "${ptx_pkg_dir}/.${pkg_label}" ]; then
 	ln -sfT $(basename "${pkg_pkg_dir}") "${ptx_pkg_dir}/.${pkg_label}"
