@@ -42,7 +42,7 @@ export -f ptxd_make_world_hash
 # perform sanity check
 #
 ptxd_make_world_prepare_sanity_check() {
-    if [ "${pkg_conf_tool}" = "autoconf" -a \! -x "${pkg_conf_dir_abs}/configure" ]; then
+    if [ "${pkg_conf_tool}" = "autoconf" -a ! -x "${pkg_conf_dir_abs}/configure" ]; then
 	cat >&2 <<EOF
 
 error: 'configure' not found or not executable in:
@@ -50,7 +50,7 @@ error: 'configure' not found or not executable in:
 
 EOF
 	exit 1
-    elif [ "${pkg_conf_tool}" = "cmake" -a \! -e "${pkg_conf_dir_abs}/CMakeLists.txt" ]; then
+    elif [ "${pkg_conf_tool}" = "cmake" -a ! -e "${pkg_conf_dir_abs}/CMakeLists.txt" ]; then
 	cat >&2 <<EOF
 
 error: 'CMakeLists.txt' not found in:
@@ -61,7 +61,7 @@ EOF
     elif [ \( "${pkg_conf_tool}" = "qmake" -o "${pkg_conf_tool}" = "perl" \) -a "${pkg_type}" != "target" ]; then
 	cat >&2 <<EOF
 
-error: only ${pkg_conf_tool} taget packages are supported
+error: only ${pkg_conf_tool} target packages are supported
 
 EOF
 	exit 1
@@ -69,7 +69,6 @@ EOF
 }
 export -f ptxd_make_world_prepare_sanity_check
 
-
 #
 # prepare for cmake based pkgs
 #
@@ -81,7 +80,8 @@ ptxd_make_world_prepare_cmake() {
 	"${pkg_conf_env}" \
 	cmake \
 	"${pkg_conf_opt}" \
-	"${pkg_conf_dir}"
+	"${pkg_conf_dir}" &&
+    ptxd_make_world_compile_commands_filter
 }
 export -f ptxd_make_world_prepare_cmake
 
@@ -149,12 +149,59 @@ ptxd_make_world_prepare_meson() {
 	"${pkg_path}" \
 	"${pkg_env}" \
 	"${pkg_conf_env}" \
-	meson \
+	meson setup \
 	"${pkg_conf_opt}" \
-	"${pkg_conf_dir}"
+	"${pkg_conf_dir}" &&
+    ptxd_make_world_compile_commands_filter
 }
 export -f ptxd_make_world_prepare_meson
 
+#
+# prepare for cargo based pkgs
+#
+ptxd_make_world_prepare_cargo_check() {
+    local arg cargo_lock_md5 crate workspace
+    local -a tmp
+    local pkg_makefile_cargo="${pkg_makefile%.make}.cargo.make"
+
+    if [ -z "${pkg_cargo_lock}" ]; then
+	return
+    fi
+
+    if [ -z "${pkg_cargo_lock_md5}" ]; then
+	ptxd_bailout "Cargo dependency config is missing!" \
+	    "Run 'ptxdist cargosync ${pkg_label}' to generate '$(ptxd_print_path ${pkg_makefile_cargo})'."
+    fi
+    tmp=( $(cd "${pkg_dir}" && md5sum "${pkg_cargo_lock}" 2> /dev/null) )
+    if [ "${tmp[0]}" != "${pkg_cargo_lock_md5}" ]; then
+	echo "|${tmp[0]}|${pkg_cargo_lock_md5}|"
+	ptxd_bailout "${pkg_cargo_lock} has changed!" \
+	    "Run 'ptxdist cargosync ${pkg_label}' to regenerate '$(ptxd_print_path ${pkg_makefile_cargo})'."
+    fi &&
+    ptxd_in_path PTXDIST_PATH_SCRIPTS vendor-cargo-workspace-package &&
+    vendor_cargo_workspace_package="${ptxd_reply}" &&
+
+    find "${pkg_cargo_home}/source" -maxdepth 1 -type l | while read link; do
+	crate="$(readlink "${link}")" &&
+	crate="${crate#../workspaces/}" &&
+	workspace="${pkg_cargo_home}/workspaces/${crate%%/*}" &&
+	crate="${pkg_cargo_home}/workspaces/${crate}" &&
+	if [ -e "${link}/Cargo.toml.orig" ]; then
+	    # already handed during a previous prepare run
+	    continue
+	fi &&
+	cp "${crate}/Cargo.toml" "${crate}/Cargo.toml.orig" &&
+	"${vendor_cargo_workspace_package}" \
+	    --input "${crate}/Cargo.toml.orig" \
+	    --output "${crate}/Cargo.toml" \
+	    --workspace "${workspace}/Cargo.toml" || {
+	    rm "${link}/Cargo.toml.orig"
+	    return
+	}
+    done
+}
+export -f ptxd_make_world_prepare_cargo_check
+
 ptxd_make_world_prepare_init() {
     # delete existing build_dir
     if [ -n "${pkg_build_oot}" ]; then
@@ -181,7 +228,8 @@ ptxd_make_world_prepare() {
 	return
     fi
 
-    ptxd_make_world_prepare_init || return
+    ptxd_make_world_prepare_init &&
+    ptxd_make_world_prepare_cargo_check|| return
 
     case "${pkg_conf_tool}" in
 	cmake|meson)
@@ -190,8 +238,8 @@ ptxd_make_world_prepare() {
 	    fi
 	    ;;
 	qmake)
-	    if ! [[ " ${pkg_build_deps} " =~ ' 'qt[45]' ' ]]; then
-		ptxd_bailout "'${pkg_label}' uses 'qmake' but does not select 'qt4' or 'qt5'"
+	    if ! [[ " ${pkg_build_deps} " =~ ' 'qt[56]' ' ]]; then
+		ptxd_bailout "'${pkg_label}' uses 'qmake' but does not select 'qt5' or 'qt6'"
 	    fi
 	    ;;
 	perl)
@@ -215,7 +263,7 @@ ptxd_make_world_prepare() {
 	autoconf|cmake|qmake|kconfig|perl|meson)
 	    cd -- "${pkg_build_dir}" &&
 	    ptxd_make_world_prepare_"${pkg_conf_tool}" ;;
-	python|python3|scons)
+	python|python3|scons|cargo)
 	    : ;; # nothing to do
 	"NO") echo "prepare stage disabled." ;;
 	"")   echo "No prepare tool found. Do nothing." ;;
